/* DianaApplet.java
 *
 * created: Sun Oct 18 1998
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1998,1999  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/pathsoft/repository/artemis_applet_impl/DianaApplet.java,v 1.3 2001/06/28 12:07:59 kmr Exp $
 */

import diana.*;
import diana.sequence.*;
import diana.components.*;

import uk.ac.sanger.pathogens.*;
import uk.ac.sanger.pathogens.embl.EntryInformation;
import uk.ac.sanger.pathogens.embl.QualifierInfoVector;
import uk.ac.sanger.pathogens.embl.QualifierInfoException;
import uk.ac.sanger.pathogens.embl.InvalidRelationException;
import uk.ac.sanger.pathogens.embl.DocumentEntryFactory;
import uk.ac.sanger.pathogens.OutOfRangeException;
import uk.ac.sanger.pathogens.embl.ReadEvent;
import uk.ac.sanger.pathogens.embl.ReadListener;

import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 *  This applet displays a button for each embl entry given as a parameter.
 *  When a button is pressed the entry is displayed in a
 *  diana.components.EntryEdit component.  The entries should be listed in the
 *  "entries" parameter.  If the sequence is in a file (on the server) called
 *  a123.seq and the features of the entry are in a file called a123.tab then
 *  the two can be combined by giving the entries parameter the value:
 *  "a123.seq +a123.tab".
 *
 *  @author Kim Rutherford
 *  @version $Id: DianaApplet.java,v 1.3 2001/06/28 12:07:59 kmr Exp $
 **/

public class DianaApplet extends java.applet.Applet {
  /**
   *  Initialise this DianaApplet applet.
   **/
  public void init () {
    final Font font = Options.getOptions ().getFont ();

    setFont (font);

    setLayout (new BorderLayout ());

    errors = new Label ("");
    add (errors, "South");

    final StringTokenizer tokeniser =
      new StringTokenizer (getParameter ("entries"), " ");

    // the base entry - must contain a sequence
    String entry_name = null;
    // other entries that will be load into the same EntryEdit component
    StringVector additional_entries = null;

    if (tokeniser.countTokens () == 0) {
      final Label message = new Label ("no entries available");

      add (message, "Center");
    } else {
      if (tokeniser.countTokens () == 1) {
        final Label message = new Label ("loading entry - please wait");

        add (message, "North");

        final String new_entry_name = tokeniser.nextToken ();

        try {
          openNewEntryEdit (new_entry_name, null);
        } catch (OutOfRangeException _) {
          new MessageFrame ("load failed: there is an out of " +
                            "range feature location");
        }

        message.setText ("loading entry - done");
      } else {

        scrollpane = new ScrollPane (ScrollPane.SCROLLBARS_AS_NEEDED);

        panel = new Panel ();
        panel.setLayout (new GridLayout (0, 2, 15, 5));

        scrollpane.add (panel);

        add (scrollpane, "Center");

        while (tokeniser.hasMoreTokens ()) {
          final String new_entry_name = tokeniser.nextToken ();

          if (new_entry_name.startsWith ("+")) {
            additional_entries.add (new_entry_name.substring (1));
          } else {
            entry_name = new_entry_name;

            final String short_entry_name;

            final int last_slash_index = entry_name.lastIndexOf ('/');

            if (last_slash_index == -1) {
              short_entry_name = entry_name;
            } else {
              short_entry_name = entry_name.substring (last_slash_index + 1);
            }

            final Button new_button = new Button (short_entry_name);

            panel.add (new_button);

            additional_entries = new StringVector ();

            final String entry_name_copy = entry_name;
            final StringVector additional_entries_copy = additional_entries;

            new_button.addActionListener (new ActionListener () {
              public void actionPerformed (ActionEvent e) {
                try {
                  openNewEntryEdit (entry_name_copy, additional_entries_copy);
                } catch (OutOfRangeException _) {
                  new MessageFrame ("load failed: there is an out of " +
                                    "range feature location");
                }
              }
            });
          }

          message = new Label ("Press a button to view an embl entry");
          add (message, "North");
        }
      }
    }
  }

  /**
   *  Create and return a new EntryEdit object.
   *  @param entry_name The name of the entry to read from the server.  It
   *    must be in the same directory as the java code.
   *  @param additional_entries A Vector containing the names of the other
   *    entries that should be added to the same EntryEdit object as entry_name
   *  @return A new EntryEdit object or null if the load fails.
   **/
  private EntryEdit openNewEntryEdit (String entry_name,
                                      StringVector additional_entries)
      throws OutOfRangeException {
    final uk.ac.sanger.pathogens.embl.Entry new_embl_entry =
      readEntry (entry_name);

    if (new_embl_entry == null) {
      return null;
    }

    final Bases embl_bases = new Bases (new_embl_entry.getSequence ());

    final EntryGroup entry_group = new SimpleEntryGroup (embl_bases);

    final Entry new_entry =
      new Entry (entry_group.getBases (), new_embl_entry);

    entry_group.add (new_entry);

    final EntryEdit entry_edit = new EntryEdit (entry_group);

    if (additional_entries != null) {
      for (int i = 0 ; i < additional_entries.size () ; ++i) {
        final uk.ac.sanger.pathogens.embl.Entry additional_embl_entry =
          readEntry (additional_entries.elementAt (i));

        if (additional_embl_entry == null) {
          entry_edit.dispose ();
          return null;
        } else {
          final Entry additional_entry =
            new Entry (entry_group.getBases (), additional_embl_entry);

          entry_group.add (additional_entry);
        }
      }
    }

    entry_edit.setVisible (true);

    errors.setText ("");

    return entry_edit;
  }


  /**
   *  Read and return an embl.Entry object from the server.
   *  @param entry_name The name of the entry to read from the server.  It
   *    must be in the same directory as the java code.
   *  @return A new Entry object or null if the load fails.
   **/
  private uk.ac.sanger.pathogens.embl.Entry readEntry (String entry_name) {
    try {
      errors.setText ("reading " + entry_name + " ...");

      final ReadListener listener =
        new ReadListener () {
          public void notify (final ReadEvent event) {
            // do nothing for now
          }
        };

      if (entry_name.indexOf ("/") == -1) {
        // absolute path
        final Document parent_document =
          new URLDocument (getDocumentBase ()).getParent ();
        final Document document = parent_document.append (entry_name);

        return DocumentEntryFactory.makeDocumentEntry (document, listener);
      } else {
        // relative path
        final Document document =
          new URLDocument (new URL (getDocumentBase ().getProtocol (),
                                    getDocumentBase ().getHost (),
                                    getDocumentBase ().getPort (),
                                    entry_name));

        return DocumentEntryFactory.makeDocumentEntry (document, listener);
      }

    } catch (IOException e) {
      errors.setText (e.toString ());
      System.err.println (e);
      return null;
    }
  }

  private Label message;
  private Label errors;
  private Canvas canvas;
  private ScrollPane scrollpane;
  private Panel panel;
}
